@@ -65,18 +65,19 @@ class SingleExperimentRunner:
         if self.YR.args.reproduce_results:
             self.reproduce_results(self.YR)
         else:
-            self.run_new_experiment()
+            self.run_new_experiment(self.YR)
 
     def start_experiment(self, args):
         api_parser = getattr(api_parsers, "API"+args.training_method)(args, self.pytorch_getter, self.global_db_path)
         run_output = api_parser.run()
+        self.eval_record_group_dicts = api_parser.get_eval_record_name_dict(return_all=True)
         del api_parser.tester_obj
         del api_parser.trainer
         del api_parser
         return run_output
 
-    def run_new_experiment(self):
-        args, _, args.dict_of_yamls = self.YR.load_yamls(**self.determine_where_to_get_yamls(self.YR.args), max_merge_depth=float('inf'))
+    def run_new_experiment(self, YR):
+        args, _, args.dict_of_yamls = YR.load_yamls(**self.determine_where_to_get_yamls(YR.args), max_merge_depth=float('inf'))
         return self.start_experiment(args)
 
     def reproduce_results(self, YR):
@@ -120,7 +121,7 @@ class SingleExperimentRunner:
         parser.add_argument("--experiment_name", type=str, required=True)
         parser.add_argument("--resume_training", action="store_true")
         parser.add_argument("--evaluate", action="store_true")
-        parser.add_argument("--splits_to_eval", nargs="+", type=str, default=None)
+        parser.add_argument("--splits_to_eval", nargs="+", type=str, default=["val"])
         parser.add_argument("--reproduce_results", type=str, default=None)
         for c in self.config_foldernames:
             parser.add_argument("--%s" % c, nargs="+", type=str, required=False, default=["default"])
